@@ -58,6 +58,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"rnng_aio_stop", (DL_FUNC) &rnng_aio_stop, 1},
   {"rnng_aio_stream_in", (DL_FUNC) &rnng_aio_stream_in, 3},
   {"rnng_aio_unresolv", (DL_FUNC) &rnng_aio_unresolv, 0},
+  {"rnng_clock", (DL_FUNC) &rnng_clock, 0},
   {"rnng_close", (DL_FUNC) &rnng_close, 1},
   {"rnng_ctx_close", (DL_FUNC) &rnng_ctx_close, 1},
   {"rnng_ctx_open", (DL_FUNC) &rnng_ctx_open, 1},
@@ -66,6 +67,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"rnng_ctx_send", (DL_FUNC) &rnng_ctx_send, 4},
   {"rnng_ctx_send_aio", (DL_FUNC) &rnng_ctx_send_aio, 3},
   {"rnng_ctx_set", (DL_FUNC) &rnng_ctx_set, 4},
+  {"rnng_device", (DL_FUNC) &rnng_device, 2},
   {"rnng_dial", (DL_FUNC) &rnng_dial, 2},
   {"rnng_dialer_close", (DL_FUNC) &rnng_dialer_close, 1},
   {"rnng_dialer_create", (DL_FUNC) &rnng_dialer_create, 2},
@@ -81,13 +83,15 @@ static const R_CallMethodDef CallEntries[] = {
   {"rnng_messenger", (DL_FUNC) &rnng_messenger, 1},
   {"rnng_ncurl", (DL_FUNC) &rnng_ncurl, 4},
   {"rnng_ncurl_aio", (DL_FUNC) &rnng_ncurl_aio, 4},
-  {"rnng_protocol_open", (DL_FUNC) &rnng_protocol_open, 1},
+  {"rnng_protocol_open", (DL_FUNC) &rnng_protocol_open, 2},
+  {"rnng_random", (DL_FUNC) &rnng_random, 0},
   {"rnng_recv", (DL_FUNC) &rnng_recv, 4},
   {"rnng_recv_aio", (DL_FUNC) &rnng_recv_aio, 2},
   {"rnng_scm", (DL_FUNC) &rnng_scm, 0},
   {"rnng_send", (DL_FUNC) &rnng_send, 4},
   {"rnng_send_aio", (DL_FUNC) &rnng_send_aio, 3},
   {"rnng_serial", (DL_FUNC) &rnng_serial, 1},
+  {"rnng_sleep", (DL_FUNC) &rnng_sleep, 1},
   {"rnng_socket_set", (DL_FUNC) &rnng_socket_set, 4},
   {"rnng_stream_close", (DL_FUNC) &rnng_stream_close, 1},
   {"rnng_stream_dial", (DL_FUNC) &rnng_stream_dial, 2},
