@@ -172,6 +172,9 @@ nanotest(is_nul_byte(as.raw(0L)) && !is_nul_byte(NULL))
 nanotest(!is_error_value(1L))
 nanotestwn(messenger("invalidURL"))
 nanotestn(tryCatch(.mirai_scm(), error = function(e) 0L))
+nanotest(is.double(mclock()))
+nanotest(is.null(msleep(1)))
+nanotest(is.double(random()))
 
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
